local ai = {}

local bfs = require('bfs')

-- ai agent
-- moves 3 squares
-- sees 6 manhattan squares
-- kills player units on touch

-- basic strategy:
-- if we can touch unit and kill it, do that
-- if there's multiple units, chose furthest one
-- if we can't reach them, go to the closest unit we see
-- if we don't see any units, go to where we last saw an unit
-- if we're there or didn't see anything yet - sit still

ai.move_distance = 2
ai.see_distance = 6

local log = function() end
--local log = print

function ai.think(self, room)
	log('ai.think start at '..tostring(self.pos))

	local d = ai.move_distance
	local move = false
	-- try to kill reachable units
	room.for_each_unit(function(u)
		-- do a cheaper distance check first
		if manhattan_distance(self.pos, u.pos) <= ai.move_distance+1 then
			log('found unit nearby at '..tostring(u.pos))
			-- do a full pathfinding
			log('bfs path from '..tostring(self.pos)..' to '..tostring(u.pos))

			local function is_walkable(pos)
				if eql_pos(pos, u.pos) then
					return true
				end
				return room.is_walkable(pos)
			end

			local path = bfs.path(self.pos, u.pos, is_walkable)
			log('path len '..#path)
			if #path <= ai.move_distance then
				log('close enough!')
				if self.path ~= nil and #self.path > #path then
					log('previous target was better')
				else
					self.alarmed = true
					self.path = path
					self.next_pos = path[#path]
					move = true
					log('going for the kill')
				end
			else
				log('too far...')
			end
		end
	end)

	-- look for visible units
	log('maybe looking for visible units')
	if not move then
		log('ok, really looking')
		local vis = bfs.visible(self.pos, room.is_opaque, ai.see_distance)
		log('seeing '..#vis..' visible tiles')
		self.visible = vis
		local closest_distance = 1000000
		local closest_unit = nil
		for _,tile in ipairs(vis) do
			room.for_each_unit_at_pos(tile, function(u)
				log('seeing an unit!')
				local d = manhattan_distance(tile, self.pos)
				if not closest_unit or d < closest_distance then
					closest_distance = d
					closest_unit = u
					log('he is the closest one so far! at '..tostring(u.pos))
					self.last_seen = tile
					self.alarmed = true
				else
					log('screw it, there is one closer')
				end
			end)
		end

		if closest_unit ~= nil then
			log('going towards closest unit at '..tostring(closest_unit.pos))

			local function is_walkable(pos)
				if eql_pos(pos, closest_unit.pos) then
					return true
				end
				return room.is_walkable(pos)
			end

			local path = bfs.path(self.pos, closest_unit.pos, is_walkable)
			-- take last ai.move_distance steps from the path
			if #path > 0 then
				self.path = {}
				for i=#path-ai.move_distance+1,#path do
					if i < 1 then
						log("oops, i'm stupid!")
					else
						table.insert(self.path, path[i])
					end
				end
				self.next_pos = path[#path]
				move = true
			end
		end
	end

	-- go towards last seen unit position
	if not move and self.last_seen then
		self.alarmed = false
		log('going towards last seen unit position at '..tostring(self.last_seen))
		local path = bfs.path(self.pos, self.last_seen, room.is_walkable)
		-- todo: check if last_seen has no unit standing there?
		-- take last d steps from the path
		self.path = {}
		for i=#path-ai.move_distance+1,#path do
			table.insert(self.path, path[i])
		end
		self.next_pos = path[#path]
		move = true
	end

	if not move then
		log("i'm bored... will stand still")
		self.next_pos = pos
	end

	self.visible = nil
	return move
end

function ai.update_visibility(self, room)
	local vis = bfs.visible(self.pos, room.is_opaque, ai.see_distance)
	self.visible = vis
end

return ai
