local rules = {}

-- puzzlescript inspired declaration language

rules.objects = {
	player = {legend = 'a', sprite = 'player', opaque = false, tint = rgba(1.0, 1.0, 1.0), layer=12},
	brick = {legend = 'b', sprite = 'box', opaque = true, tint = rgba(1.0, 1.0, 1.0), layer=10},
	wall = {legend = '#', sprite = 'wall', opaque = true, tint = rgba(1.0, 1.0, 1.0), layer=10},
	hole = {legend = 'h', sprite = 'hole', opaque = false, tint = rgba(1.0, 1.0, 1.0)},
	enemy = {legend = 'e', sprite = 'enemy_green', opaque = false, tint = rgba(1.0, 1.0, 1.0), layer=11},
}

rules.desc = {
	-- player can push brick
	{'>', 'player', 'brick', '>', '>'},

	-- player can push player
	{'>', 'player', 'player', '>', '>'},

	-- player can push enemy
	{'>', 'player', 'enemy', '>', '>'},

	-- brick can push enemy
	{'>', 'brick', 'enemy', '>', '>'},

	-- brick can push player
	{'>', 'brick', 'player', '>', '>'},

	-- brick can't push brick!
	--{'>', 'brick', 'brick', '>', '>'},
	
	-- enemy can push brick
	{'>', 'enemy', 'brick', '>', '>'},

	-- enemy can push player
	{'>', 'enemy', 'player', '>', '>'},

	-- enemy can push enemy
	{'>', 'enemy', 'enemy', '>', '>'},

	-- brick can move into a hole
	{'>', 'brick', 'hole', '>', ''},

	-- enemy can move into a hole
	{'>', 'enemy', 'hole', '>', ''}
}

---

function rules.parse_level(level)
	local width = nil
	local height = #level
	local objs = {}
	for y,row in ipairs(level) do
		if width == nil then
			width = #row
		end

		-- iterate over chars
		local x = 0
		for c in row:gmatch('.') do
			for id,obj in pairs(rules.objects) do
				if c == obj.legend then
					local glow = nil
					if obj.glow then
						glow = sprsheet.get_handle(obj.glow)
					end
					table.insert(objs, {
						id = id,
						sprite = sprsheet.get_handle(obj.sprite),
						glow = glow,
						tint = lerp(obj.tint, obj.tint, 1),
						layer = obj.layer,
						opaque = obj.opaque,
						pos = vec2(x, y-1),
						rot = 0
					})
				end
			end
			x = x + 1
		end
	end

	return objs, width, height
end

return rules
